#!/usr/bin/env python
"""
This script queries the WMStats server and creates a summary of success, failures,
type of failures, files skipped per agent; as well as an overview of failures
per task
"""
from __future__ import print_function, division

from future.utils import viewitems

from future import standard_library
standard_library.install_aliases()
import http.client
import json
import os
import sys
from pprint import pformat


def getWMStatsData(workflow):
    url = 'cmsweb.cern.ch'
    headers = {"Accept": "application/json", "Content-type": "application/json"}
    conn = http.client.HTTPSConnection(url, cert_file=os.getenv('X509_USER_PROXY'), key_file=os.getenv('X509_USER_PROXY'))
    urn = "/wmstatsserver/data/request/%s" % workflow
    conn.request("GET", urn, headers=headers)
    r2 = conn.getresponse()
    request = json.loads(r2.read())["result"][0]

    return request


def main():
    if len(sys.argv) != 2:
        print("A workflow name must be provided!")
        sys.exit(1)

    wf = sys.argv[1]
    data = getWMStatsData(wf)
    if 'AgentJobInfo' not in data[wf]:
        print("Request doesn't have a `AgentJobInfo` key, nothing to be done")
        sys.exit(2)
    data = data[wf]['AgentJobInfo']

    summary = {}
    for agent in data:
        print("\nChecking AgentJobInfo for: %s" % agent)
        print("  Skipped files: %s" % pformat(data[agent]['skipped']))
        print("  Overall agent status:\t\t %s" % data[agent]['status'])

        for task, values in viewitems(data[agent]['tasks']):
            if values['jobtype'] not in ['Production', 'Processing', 'Merge', 'Harvest']:
                # print("Skipping task type %s, for %s" % (values['jobtype'], task))
                continue
            taskName = task.split('/')[-1]
            if 'status' not in values:
                print("'status' key not available under task: %s" % taskName)
                continue
            if 'failure' in values['status']:
                taskName = task.split('/')[-1]
                summary.setdefault(taskName, 0)
                summary[taskName] += sum(values['status']['failure'].values())

    print("\nSummary of failures for workflow: %s" % wf)
    print(pformat(summary))

    sys.exit(0)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
"""
_Subscription_

workflow + fileset = subscription
"""

from builtins import range
import copy

from WMCore.DataStructs.Pickleable import Pickleable
from WMCore.DataStructs.Fileset import Fileset

class Subscription(Pickleable, dict):
    def __init__(self, fileset = None, workflow = None,
                 split_algo = "FileBased", type = "Processing"):
        if fileset == None:
            fileset = Fileset()

        self.setdefault('fileset', fileset)
        self.setdefault('workflow', workflow)
        self.setdefault('type', type)

        self.setdefault('split_algo', split_algo)

        self.available = Fileset(name=fileset.name,
                                 files = fileset.getFiles())

        self.acquired = Fileset(name='acquired')
        self.completed = Fileset(name='completed')
        self.failed = Fileset(name='failed')

    def name(self):
        return self.getWorkflow().name.replace(' ', '') + '_' + \
                    self.getFileset().name.replace(' ', '')

    def getWorkflow(self):
        return self["workflow"]

    def workflowName(self):
        if self["workflow"] == None:
            return "Unknown"
        return self["workflow"].name

    def workflowType(self):
        if self["workflow"] == None:
            return "Unknown"
        return self["workflow"].wfType

    def taskName(self):
        if self['workflow'] == None:
            return "Unknown"
        return self['workflow'].task

    def owner(self):
        if self['workflow'] == None:
            return 'Unknown'
        return self['workflow'].owner

    def getFileset(self):
        return self['fileset']

    def acquireFiles(self, files = [], size=1):
        """
        Return the files acquired
        """
        self.acquired.commit()
        self.available.commit()
        self.failed.commit()
        self.completed.commit()
        retval = []
        if len(files):
            for i in files:
                # Check each set, instead of elif, just in case something has
                # got out of synch
                if i in self.available.files:
                    self.available.files.remove(i)
                if i in self.failed.files:
                    self.failed.files.remove(i)
                if i in self.completed.files:
                    self.completed.files.remove(i)
                self.acquired.addFile(i)
        else:
            if len(self.available.files) < size or size == 0:
                size = len(self.available.files)
            for i in range(size):
                self.acquired.addFile(self.available.files.pop())

        return self.acquired.listNewFiles()

    def completeFiles(self, files):
        """
        Return the number of files complete
        """
        self.acquired.commit()
        self.available.commit()
        self.failed.commit()
        self.completed.commit()
        for i in files:
            # Check each set, instead of elif, just in case something has
            # got out of synch
            if i in self.available.files:
                self.available.files.remove(i)
            if i in self.failed.files:
                self.failed.files.remove(i)
            if i in self.acquired.files:
                self.acquired.files.remove(i)
            self.completed.addFile(i)

    def failFiles(self, files):
        """
        Return the number of files failed
        """
        self.acquired.commit()
        self.available.commit()
        self.failed.commit()
        self.completed.commit()
        for i in files:
            # Check each set, instead of elif, just in case something has
            # got out of synch
            if i in self.available.files:
                self.available.files.remove(i)
            if i in self.completed.files:
                self.completed.files.remove(i)
            if i in self.acquired.files:
                self.acquired.files.remove(i)
            self.failed.addFile(i)

    def filesOfStatus(self, status=None, doingJobSplitting = False):
        """
        _filesOfStatus_

        Return a Set of File objects that are associated with the subscription
        and have a particular status.
        """
        status = status.title()
        if status == 'Available':
            return self.available.getFiles(type='set') - \
            (self.acquiredFiles() | self.completedFiles() | self.failedFiles())
        elif status == 'Acquired':
            return self.acquired.getFiles(type='set')
        elif status == 'Completed':
            return self.completed.getFiles(type='set')
        elif status == 'Failed':
            return self.failed.getFiles(type='set')

    def availableFiles(self, limit = None, doingJobSplitting = False):
        """
        _availableFiles_

        Return a Set of files that are available for processing
        (e.g. not already in use)
        """
        if limit:
            return list(self.filesOfStatus(status = "Available", doingJobSplitting = doingJobSplitting))[:limit]
        else:
            return self.filesOfStatus(status = "Available", doingJobSplitting = doingJobSplitting)

    def acquiredFiles(self):
        """
        Set of files marked as acquired.
        """
        return self.filesOfStatus(status = "Acquired")

    def completedFiles(self):
        """
        Set of files marked as completed.
        """
        return self.filesOfStatus(status = "Completed")

    def failedFiles(self):
        """
        Set of files marked as failed.
        """
        return self.filesOfStatus(status = "Failed")



#!/usr/bin/env python
# encoding: utf-8
"""
CouchObject.py

Created by Dave Evans on 2010-07-14.
Copyright (c) 2010 Fermilab. All rights reserved.
"""
from __future__ import print_function

from builtins import str
import json as jsonlib

import WMCore.Database.CMSCouch as CMSCouch
import WMCore.GroupUser.Decorators as Decorators


class CouchConnectionError(Exception):
    """docstring for CouchConnectionError"""

    def __init__(self, arg):
        super(CouchConnectionError, self).__init__()
        self.arg = arg


class CouchObject(dict):
    """
    Base class for dictionary derived couch documents for this package.
    (May even be generally useful...)

    Essentially a specialised dict class that has attributes needed to talk to Couch
    via the CMSCouch API.

    This class is expected to be overridden and the derived class must override the document_id
    property to generate the document id from the data within the dictionary.
    This class assumes some simple mapping is used to generate the document ID as a property, rather
    on doing anything too fancy
    """

    def __init__(self):
        dict.__init__(self)
        self.couch = None
        self.cdb_server = None
        self.cdb_database = None
        self.cdb_url = None
        self.cdb_document_id = None
        self.cdb_document_data = "CouchObject"

    connected = property(lambda x: x.couch is not None)
    json = property(lambda x: jsonlib.dumps(dict(x)))
    # doc_id default returns None, which signals use couch-generated doc id
    document_id = property(lambda x: None)

    def setCouch(self, url, database):
        """
        _setCouch_

        Set the contant info for the couch database
        """
        self.cdb_url = url
        self.cdb_database = database

    def connect(self):
        """
        _connect_

        Initialise the couch database connection for this object.
        This gets called automagically by the requireConnected decorator
        """
        if self.connected:
            return
        if self.cdb_url is None:
            msg = "url for couch service not provided"
            raise CouchConnectionError(msg)
        if self.cdb_database is None:
            msg = "database name for couch service not provided"
            raise CouchConnectionError(msg)
        try:
            self.cdb_server = CMSCouch.CouchServer(self.cdb_url)
            self.couch = self.cdb_server.connectDatabase(self.cdb_database)
        except Exception as ex:
            msg = "Exception instantiating couch services for :\n"
            msg += " url = %s\n database = %s\n" % (self.cdb_url, self.cdb_database)
            msg += " Exception: %s" % str(ex)
            print(msg)
            raise CouchConnectionError(msg)

    @Decorators.requireConnection
    def create(self):
        """
        _create_

        Create the couch document for this object.
        """
        if not self.couch.documentExists(self.document_id):
            couchDoc = CMSCouch.Document(self.document_id, {self.cdb_document_data: dict(self)})
            self.couch.commitOne(couchDoc)

    @Decorators.requireConnection
    def get(self):
        """
        _get_

        given the doc_id generated from the derived class, get the document and update
        the data in the dictionary from it

        """
        if not self.couch.documentExists(self.document_id):
            raise RuntimeError("Document: %s not found" % self.document_id)

        doc = self.couch.document(self.document_id)
        data = doc.get(self.cdb_document_data, {})
        dict.update(self, data)
        return

    @Decorators.requireConnection
    def drop(self):
        """
        _drop_

        remove the doc representing this object, possibly with a chained wipeout
        of all other docs referencing it by owner or group or whatever, if thats
        needed, override in the classes derived from this
        """
        if not self.couch.documentExists(self.document_id):
            raise RuntimeError("Document: %s not found" % self.document_id)
        self.couch.delete_doc(self.document_id)
